<?php declare(strict_types=1);

/*
 *     Pushbullet Notifier [sikofitt/pushbullet-notifier]
 *     Copyright (C) 2017  http://blog.rewiv.com R. Eric Wheeler <sikofitt@gmail.com>
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Pushbullet\Console;

use Monolog\Handler\StreamHandler;
use Monolog\Handler\SyslogHandler;
use Monolog\Logger;
use Psr\Container\ContainerInterface;
use Sikofitt\Pushbullet\Console\Command\PushbulletDevicesCommand;
use Sikofitt\Pushbullet\Console\Command\PushbulletNotifierCommand;
use Sikofitt\Pushbullet\DependencyInjection\Container;
use Sikofitt\Pushbullet\DependencyInjection\PushBulletExtension;
use Symfony\Component\Console\Application as SymfonyApplication;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\DependencyInjection\Exception\InvalidArgumentException;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBag;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

class Application extends SymfonyApplication
{
    protected const APP_VERSION = '0.0.1';
    protected const APP_NAME = 'Pushbullet Notifier';

    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @param array $parameters Parameters to set at runtime.
     *
     * @throws \LogicException
     */
    public function __construct(array $parameters = [])
    {
        $this->buildContainer($parameters);

        $this->add(new PushbulletNotifierCommand($this->container));
        $this->add(new PushbulletDevicesCommand($this->container));

        parent::__construct(self::APP_NAME, self::APP_VERSION);
    }
    /**
     * @throws \Symfony\Component\DependencyInjection\Exception\InvalidArgumentException
     * @return null|string
     */
    public function getRootDir(): ?string
    {
        try {
            return $this->container->getParameter('app.root');
        } catch (InvalidArgumentException $i) {
            return null;
        }
    }

    /**
     * @param string $rootDir
     */
    public function setRootDir(string $rootDir): void
    {
        $this->container->setParameter('app.root', $rootDir);
    }

    /**
     * Runs the current application.
     *
     * @param InputInterface $input An Input instance
     * @param OutputInterface $output An Output instance
     *
     * @throws \Exception When running fails. Bypass this when {@link setCatchExceptions()}.
     * @return int 0 if everything went fine, or an error code
     *
     */
    public function run(
      InputInterface $input = null,
      OutputInterface $output = null
    ): int {
        return parent::run(
            $input,
          $output
        ); // TODO: Change the autogenerated stub
    }

    /**
     * @return \Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface
     */
    public function getParameterBag(): ParameterBagInterface
    {
        return $this->container->getParameterBag();
    }

    /**
     * @param \Psr\Container\ContainerInterface $container
     *
     * @return $this
     */
    public function setContainer(ContainerInterface $container): Application
    {
        $this->container = $container;

        return $this;
    }

    private function buildContainer(array $parameters): void
    {
        $extension = new PushBulletExtension();
        $parameterBag = new ParameterBag($parameters);
        $this->container = new Container($parameterBag);
        $this->container->registerExtension($extension);
        $this->container->loadFromExtension($extension->getAlias());
        $this->container->setParameter('app.root', dirname(__DIR__, 4));

        $logFile = sprintf('%s/log/pushbullet.log', $this->container->getParameter('app.root'));

        $handlers[] = new StreamHandler($logFile);
        $handlers[] = new SyslogHandler('pushbullet');
        $logger = new Logger('pushbullet', $handlers);
        $this->container->set('logger', $logger);

        $this->container->compile();
    }
}
