<?php declare(strict_types=1);

/*
 *     Pushbullet Notifier [sikofitt/pushbullet-notifier]
 *     Copyright (C) 2017  http://blog.rewiv.com R. Eric Wheeler <sikofitt@gmail.com>
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace Sikofitt\Pushbullet\DependencyInjection;

use \Symfony\Component\Yaml\Yaml;
use Symfony\Component\Config\Definition\Processor;
use Symfony\Component\Config\Exception\FileLocatorFileNotFoundException;
use Symfony\Component\Config\FileLocator;
use Symfony\Component\Console\Exception\RuntimeException;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Extension\Extension;
use Symfony\Component\Yaml\Exception\ParseException;

class PushBulletExtension extends Extension
{
    /**
     * Loads a specific configuration.
     *
     * @param array $configs An array of configuration values
     * @param ContainerBuilder $container A ContainerBuilder instance
     *
     * @throws \InvalidArgumentException When provided tag is not defined in
     *   this extension
     */
    public function load(array $configs, ContainerBuilder $container): void
    {
        $locator = new FileLocator([$container->getParameter('app.root').'/config']);

        try {
            $config = $locator->locate('config.yaml');
        } catch (FileLocatorFileNotFoundException $f) {
            print $f->getMessage();
            exit;
        }

        try {
            $configs = Yaml::parse(file_get_contents($config));
        } catch (ParseException $p) {
            print  sprintf('%sUnable to parse %s at line %d (stopped at "%s") in %s%s', PHP_EOL, $config, $p->getParsedLine(), $p->getSnippet(), PHP_EOL, PHP_EOL);
            exit;
        } catch (\Exception $e) {
            print PHP_EOL . $e->getMessage() . PHP_EOL . PHP_EOL;
            exit;
        }

        $configuration = new Configuration();
        $processor = new Processor();
        $config = $processor->processConfiguration($configuration, $configs);
        $object = new \stdClass();
        $object->class = new \ArrayObject();
        $object->arguments = [
          ['array']
        ];
        $container->set('pushbullet.app', $object);

        $container->setParameter('pushbullet.token', $config['token']);
        $container->setParameter('pushbullet.push_devices', $config['push_devices']);
    }

    /**
     * Returns the recommended alias to use in XML.
     *
     * This alias is also the mandatory prefix to use when using YAML.
     *
     * @return string The alias
     */
    public function getAlias()
    {
        return 'pushbullet';
    }
}
